import dedent from 'dedent';
import log from 'npmlog';
import yargs from 'yargs/yargs';
import { globalOptions } from './global-options.js';
function lernaCLI(argv, cwd) {
    const cli = yargs(argv, cwd);
    return globalOptions(cli)
        .usage('Usage: $0 <command> [options]')
        .demandCommand(1, 'A command is required. Pass --help to see all available commands and options.')
        .recommendCommands()
        .strict()
        .fail((msg, err) => {
        const actual = err || new Error(msg);
        if (actual.name !== 'ValidationError' && !actual.pkg) {
            if (/Did you mean/.test(actual.message)) {
                log.error('lerna-lite', `Unknown command "${cli.parsed.argv._[0]}"`);
            }
            log.error('lerna-lite', actual.message);
        }
        cli.exit(actual.exitCode > 0 ? actual.exitCode : 1, actual);
    })
        .alias('h', 'help')
        .alias('v', 'version')
        .wrap(cli.terminalWidth()).epilogue(dedent `
      When a command fails, all logs are written to lerna-debug.log in the current working directory.

      For more information, find our manual at https://github.com/lerna/lerna
    `);
}
export default lernaCLI;
//# sourceMappingURL=lerna-cli.js.map