import fetch from 'node-fetch';
import log from 'npmlog';
import { join } from 'node:path';
export class GitLabClient {
    constructor(token, baseUrl = 'https://gitlab.com/api/v4') {
        this.baseUrl = baseUrl;
        this.token = token;
    }
    createRelease({ owner, repo, name, tag_name: tagName, body }) {
        const releasesUrl = this.releasesUrl(owner, repo, 'releases');
        log.silly('Requesting GitLab releases', releasesUrl);
        return fetch(releasesUrl, {
            method: 'post',
            body: JSON.stringify({ name, tag_name: tagName, description: body }),
            headers: {
                'PRIVATE-TOKEN': this.token,
                'Content-Type': 'application/json',
            },
        }).then(({ ok, status, statusText }) => {
            if (!ok) {
                log.error('gitlab', `Failed to create release\nRequest returned ${status} ${statusText}`);
            }
            else {
                log.silly('gitlab', 'Created release successfully.');
            }
        });
    }
    releasesUrl(namespace, project, releaseType = 'releases') {
        return new URL(`${this.baseUrl}/${join('projects', encodeURIComponent(`${namespace}/${project}`), releaseType)}`).toString();
    }
}
//# sourceMappingURL=GitLabClient.js.map