"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.matchExpectedRPID = void 0;
const toHash_1 = require("./toHash");
const iso_1 = require("./iso");
/**
 * Go through each expected RP ID and try to find one that matches. Raises an Error if no
 */
async function matchExpectedRPID(rpIDHash, expectedRPIDs) {
    try {
        await Promise.any(expectedRPIDs.map(expected => {
            return new Promise((resolve, reject) => {
                (0, toHash_1.toHash)(iso_1.isoUint8Array.fromASCIIString(expected)).then(expectedRPIDHash => {
                    if (iso_1.isoUint8Array.areEqual(rpIDHash, expectedRPIDHash)) {
                        resolve(true);
                    }
                    else {
                        reject();
                    }
                });
            });
        }));
    }
    catch (err) {
        const _err = err;
        // This means no matches were found
        if (_err.name === 'AggregateError') {
            throw new UnexpectedRPIDHash();
        }
        // An unexpected error occurred
        throw err;
    }
}
exports.matchExpectedRPID = matchExpectedRPID;
class UnexpectedRPIDHash extends Error {
    constructor() {
        const message = 'Unexpected RP ID hash';
        super(message);
        this.name = 'UnexpectedRPIDHash';
    }
}
//# sourceMappingURL=matchExpectedRPID.js.map