"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpService = void 0;
const utils_1 = require("@standardnotes/utils");
const models_1 = require("@standardnotes/models");
const domain_core_1 = require("@standardnotes/domain-core");
const responses_1 = require("@standardnotes/responses");
const XMLHttpRequestState_1 = require("./XMLHttpRequestState");
const ErrorMessage_1 = require("../Error/ErrorMessage");
const Paths_1 = require("../Server/Auth/Paths");
class HttpService {
    constructor(environment, appVersion, snjsVersion) {
        this.environment = environment;
        this.appVersion = appVersion;
        this.snjsVersion = snjsVersion;
        this.session = null;
    }
    setCallbacks(updateMetaCallback, refreshSessionCallback) {
        this.updateMetaCallback = updateMetaCallback;
        this.refreshSessionCallback = refreshSessionCallback;
    }
    deinit() {
        this.session = null;
        this.updateMetaCallback = undefined;
        this.refreshSessionCallback = undefined;
    }
    setSession(session) {
        this.session = session;
    }
    setHost(host) {
        this.host = host;
    }
    get(path, params, authentication) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return this.runHttp({
                url: (0, utils_1.joinPaths)(this.host, path),
                params,
                verb: responses_1.HttpVerb.Get,
                authentication: authentication !== null && authentication !== void 0 ? authentication : (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value,
            });
        });
    }
    post(path, params, authentication) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return this.runHttp({
                url: (0, utils_1.joinPaths)(this.host, path),
                params,
                verb: responses_1.HttpVerb.Post,
                authentication: authentication !== null && authentication !== void 0 ? authentication : (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value,
            });
        });
    }
    put(path, params, authentication) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return this.runHttp({
                url: (0, utils_1.joinPaths)(this.host, path),
                params,
                verb: responses_1.HttpVerb.Put,
                authentication: authentication !== null && authentication !== void 0 ? authentication : (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value,
            });
        });
    }
    patch(path, params, authentication) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return this.runHttp({
                url: (0, utils_1.joinPaths)(this.host, path),
                params,
                verb: responses_1.HttpVerb.Patch,
                authentication: authentication !== null && authentication !== void 0 ? authentication : (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value,
            });
        });
    }
    delete(path, params, authentication) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return this.runHttp({
                url: (0, utils_1.joinPaths)(this.host, path),
                params,
                verb: responses_1.HttpVerb.Delete,
                authentication: authentication !== null && authentication !== void 0 ? authentication : (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value,
            });
        });
    }
    runHttp(httpRequest) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.inProgressRefreshSessionPromise) {
                yield this.inProgressRefreshSessionPromise;
                httpRequest.authentication = (_a = this.session) === null || _a === void 0 ? void 0 : _a.accessToken.value;
            }
            const request = this.createXmlRequest(httpRequest);
            if (this.__latencySimulatorMs) {
                yield (0, utils_1.sleep)(this.__latencySimulatorMs, true);
            }
            const response = yield this.runRequest(request, this.createRequestBody(httpRequest));
            if (response.meta) {
                (_b = this.updateMetaCallback) === null || _b === void 0 ? void 0 : _b.call(this, response.meta);
            }
            if (response.status === responses_1.HttpStatusCode.ExpiredAccessToken) {
                if (this.inProgressRefreshSessionPromise) {
                    yield this.inProgressRefreshSessionPromise;
                }
                else {
                    this.inProgressRefreshSessionPromise = this.refreshSession();
                    const isSessionRefreshed = yield this.inProgressRefreshSessionPromise;
                    this.inProgressRefreshSessionPromise = undefined;
                    if (!isSessionRefreshed) {
                        return response;
                    }
                }
                httpRequest.authentication = (_c = this.session) === null || _c === void 0 ? void 0 : _c.accessToken.value;
                return this.runHttp(httpRequest);
            }
            return response;
        });
    }
    refreshSession() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.session === null) {
                return false;
            }
            const response = yield this.post(Paths_1.Paths.v1.refreshSession, {
                access_token: this.session.accessToken.value,
                refresh_token: this.session.refreshToken.value,
            });
            if ((0, responses_1.isErrorResponse)(response)) {
                return false;
            }
            if (response.meta) {
                (_a = this.updateMetaCallback) === null || _a === void 0 ? void 0 : _a.call(this, response.meta);
            }
            const accessTokenOrError = domain_core_1.SessionToken.create(response.data.session.access_token, response.data.session.access_expiration);
            if (accessTokenOrError.isFailed()) {
                return false;
            }
            const accessToken = accessTokenOrError.getValue();
            const refreshTokenOrError = domain_core_1.SessionToken.create(response.data.session.refresh_token, response.data.session.refresh_expiration);
            if (refreshTokenOrError.isFailed()) {
                return false;
            }
            const refreshToken = refreshTokenOrError.getValue();
            const sessionOrError = domain_core_1.Session.create(accessToken, refreshToken, response.data.session.readonly_access);
            if (sessionOrError.isFailed()) {
                return false;
            }
            this.setSession(sessionOrError.getValue());
            this.refreshSessionCallback(this.session);
            return true;
        });
    }
    createRequestBody(httpRequest) {
        if (httpRequest.params !== undefined &&
            [responses_1.HttpVerb.Post, responses_1.HttpVerb.Put, responses_1.HttpVerb.Patch, responses_1.HttpVerb.Delete].includes(httpRequest.verb)) {
            return JSON.stringify(httpRequest.params);
        }
        return httpRequest.rawBytes;
    }
    createXmlRequest(httpRequest) {
        var _a;
        const request = new XMLHttpRequest();
        if (httpRequest.params && httpRequest.verb === responses_1.HttpVerb.Get && Object.keys(httpRequest.params).length > 0) {
            httpRequest.url = this.urlForUrlAndParams(httpRequest.url, httpRequest.params);
        }
        request.open(httpRequest.verb, httpRequest.url, true);
        request.responseType = (_a = httpRequest.responseType) !== null && _a !== void 0 ? _a : '';
        if (!httpRequest.external) {
            request.setRequestHeader('X-SNJS-Version', this.snjsVersion);
            const appVersionHeaderValue = `${models_1.Environment[this.environment]}-${this.appVersion}`;
            request.setRequestHeader('X-Application-Version', appVersionHeaderValue);
            if (httpRequest.authentication) {
                request.setRequestHeader('Authorization', 'Bearer ' + httpRequest.authentication);
            }
        }
        let contenTypeIsSet = false;
        if (httpRequest.customHeaders && httpRequest.customHeaders.length > 0) {
            httpRequest.customHeaders.forEach(({ key, value }) => {
                request.setRequestHeader(key, value);
                if (key === 'Content-Type') {
                    contenTypeIsSet = true;
                }
            });
        }
        if (!contenTypeIsSet && !httpRequest.external) {
            request.setRequestHeader('Content-Type', 'application/json');
        }
        return request;
    }
    runRequest(request, body) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                request.onreadystatechange = () => {
                    this.stateChangeHandlerForRequest(request, resolve);
                };
                request.send(body);
            });
        });
    }
    stateChangeHandlerForRequest(request, resolve) {
        var _a, _b, _c;
        if (request.readyState !== XMLHttpRequestState_1.XMLHttpRequestState.Completed) {
            return;
        }
        const httpStatus = request.status;
        const response = {
            status: httpStatus,
            headers: new Map(),
            data: {},
        };
        const responseHeaderLines = (_a = request
            .getAllResponseHeaders()) === null || _a === void 0 ? void 0 : _a.trim().split(/[\r\n]+/);
        responseHeaderLines === null || responseHeaderLines === void 0 ? void 0 : responseHeaderLines.forEach((responseHeaderLine) => {
            const parts = responseHeaderLine.split(': ');
            const name = parts.shift();
            const value = parts.join(': ');
            response.headers.set(name, value);
        });
        try {
            if (httpStatus !== responses_1.HttpStatusCode.NoContent) {
                let body;
                const contentTypeHeader = ((_b = response.headers) === null || _b === void 0 ? void 0 : _b.get('content-type')) || ((_c = response.headers) === null || _c === void 0 ? void 0 : _c.get('Content-Type'));
                if (contentTypeHeader === null || contentTypeHeader === void 0 ? void 0 : contentTypeHeader.includes('application/json')) {
                    body = JSON.parse(request.responseText);
                }
                else {
                    body = request.response;
                }
                /**
                 * v0 APIs do not have a `data` top-level object. In such cases, mimic
                 * the newer response body style by putting all the top-level
                 * properties inside a `data` object.
                 */
                if (!body.data) {
                    response.data = body;
                }
                if (!(0, utils_1.isString)(body)) {
                    Object.assign(response, body);
                }
            }
        }
        catch (error) {
            console.error(error);
        }
        if (httpStatus >= responses_1.HttpStatusCode.Success && httpStatus < responses_1.HttpStatusCode.InternalServerError) {
            if (httpStatus === responses_1.HttpStatusCode.Forbidden &&
                response.data &&
                response.data.error !== undefined) {
                ;
                response.data.error.message = ErrorMessage_1.ErrorMessage.RateLimited;
            }
            resolve(response);
        }
        else {
            const errorResponse = response;
            if (!errorResponse.data) {
                errorResponse.data = {
                    error: {
                        message: 'Unknown error',
                    },
                };
            }
            if ((0, utils_1.isString)(errorResponse.data)) {
                errorResponse.data = {
                    error: {
                        message: errorResponse.data,
                    },
                };
            }
            if (!errorResponse.data.error) {
                errorResponse.data.error = {
                    message: 'Unknown error',
                };
            }
            resolve(errorResponse);
        }
    }
    urlForUrlAndParams(url, params) {
        const keyValueString = Object.keys(params)
            .map((key) => {
            return key + '=' + encodeURIComponent(params[key]);
        })
            .join('&');
        if (url.includes('?')) {
            return url + '&' + keyValueString;
        }
        else {
            return url + '?' + keyValueString;
        }
    }
}
exports.HttpService = HttpService;
