import { ProtocolVersion } from '@standardnotes/common';
import { DecryptedItem, DecryptedPayloadInterface, NamespacedRootKeyInKeychain, RootKeyContent, RootKeyContentInStorage, RootKeyInterface } from '@standardnotes/models';
import { SNRootKeyParams } from './RootKeyParams';
/**
 * A root key is a local only construct that houses the key used for the encryption
 * and decryption of items keys. A root key extends SNItem for local convenience, but is
 * not part of the syncing or storage ecosystem—root keys are managed independently.
 */
export declare class SNRootKey extends DecryptedItem<RootKeyContent> implements RootKeyInterface {
    readonly keyParams: SNRootKeyParams;
    constructor(payload: DecryptedPayloadInterface<RootKeyContent>);
    get keyVersion(): ProtocolVersion;
    /**
     * When the root key is used to encrypt items, we use the masterKey directly.
     */
    get itemsKey(): string;
    get masterKey(): string;
    /**
     * serverPassword is not persisted as part of keychainValue, so if loaded from disk,
     * this value may be undefined.
     */
    get serverPassword(): string | undefined;
    /** 003 and below only. */
    get dataAuthenticationKey(): string | undefined;
    compare(otherKey: SNRootKey): boolean;
    /**
     * @returns Object suitable for persist in storage when wrapped
     */
    persistableValueWhenWrapping(): RootKeyContentInStorage;
    /**
     * @returns Object that is suitable for persisting in a keychain
     */
    getKeychainValue(): NamespacedRootKeyInKeychain;
}
