import { KeyParamsOrigination, ProtocolVersion } from '@standardnotes/common';
import * as Models from '@standardnotes/models';
import { ItemContent, ItemsKeyInterface } from '@standardnotes/models';
import { PureCryptoInterface } from '@standardnotes/sncrypto-common';
import { SNRootKey } from '../../Keys/RootKey/RootKey';
import { SNRootKeyParams } from '../../Keys/RootKey/RootKeyParams';
import { DecryptedParameters, EncryptedParameters, ErrorDecryptingParameters } from '../../Types/EncryptedParameters';
import { ItemAuthenticatedData } from '../../Types/ItemAuthenticatedData';
import { LegacyAttachedData } from '../../Types/LegacyAttachedData';
import { RootKeyEncryptedAuthenticatedData } from '../../Types/RootKeyEncryptedAuthenticatedData';
import { SynchronousOperator } from '../Operator';
type V004StringComponents = [version: string, nonce: string, ciphertext: string, authenticatedData: string];
type V004Components = {
    version: V004StringComponents[0];
    nonce: V004StringComponents[1];
    ciphertext: V004StringComponents[2];
    authenticatedData: V004StringComponents[3];
};
export declare class SNProtocolOperator004 implements SynchronousOperator {
    protected readonly crypto: PureCryptoInterface;
    constructor(crypto: PureCryptoInterface);
    getEncryptionDisplayName(): string;
    get version(): ProtocolVersion;
    private generateNewItemsKeyContent;
    /**
     * Creates a new random items key to use for item encryption.
     * The consumer must save/sync this item.
     */
    createItemsKey(): ItemsKeyInterface;
    /**
     * We require both a client-side component and a server-side component in generating a
     * salt. This way, a comprimised server cannot benefit from sending the same seed value
     * for every user. We mix a client-controlled value that is globally unique
     * (their identifier), with a server controlled value to produce a salt for our KDF.
     * @param identifier
     * @param seed
     */
    private generateSalt004;
    /**
     * Computes a root key given a passworf
     * qwd and previous keyParams
     * @param password - Plain string representing raw user password
     * @param keyParams - KeyParams object
     */
    computeRootKey(password: string, keyParams: SNRootKeyParams): Promise<SNRootKey>;
    /**
     * Creates a new root key given an identifier and a user password
     * @param identifier - Plain string representing a unique identifier
     * @param password - Plain string representing raw user password
     */
    createRootKey(identifier: string, password: string, origination: KeyParamsOrigination): Promise<SNRootKey>;
    /**
     * @param plaintext - The plaintext to encrypt.
     * @param rawKey - The key to use to encrypt the plaintext.
     * @param nonce - The nonce for encryption.
     * @param authenticatedData - JavaScript object (will be stringified) representing
                  'Additional authenticated data': data you want to be included in authentication.
     */
    encryptString004(plaintext: string, rawKey: string, nonce: string, authenticatedData: ItemAuthenticatedData): string;
    /**
     * @param ciphertext  The encrypted text to decrypt.
     * @param rawKey  The key to use to decrypt the ciphertext.
     * @param nonce  The nonce for decryption.
     * @param rawAuthenticatedData String representing
                  'Additional authenticated data' - data you want to be included in authentication.
     */
    private decryptString004;
    generateEncryptionNonce(): string;
    /**
     * @param plaintext  The plaintext text to decrypt.
     * @param rawKey  The key to use to encrypt the plaintext.
     */
    generateEncryptedProtocolString(plaintext: string, rawKey: string, authenticatedData: ItemAuthenticatedData): string;
    deconstructEncryptedPayloadString(payloadString: string): V004Components;
    getPayloadAuthenticatedData(encrypted: EncryptedParameters): RootKeyEncryptedAuthenticatedData | ItemAuthenticatedData | LegacyAttachedData | undefined;
    /**
     * For items that are encrypted with a root key, we append the root key's key params, so
     * that in the event the client/user loses a reference to their root key, they may still
     * decrypt data by regenerating the key based on the attached key params.
     */
    private generateAuthenticatedDataForPayload;
    private authenticatedDataToString;
    private stringToAuthenticatedData;
    generateEncryptedParametersSync(payload: Models.DecryptedPayloadInterface, key: ItemsKeyInterface | SNRootKey): EncryptedParameters;
    generateDecryptedParametersSync<C extends ItemContent = ItemContent>(encrypted: EncryptedParameters, key: ItemsKeyInterface | SNRootKey): DecryptedParameters<C> | ErrorDecryptingParameters;
    private deriveKey;
}
export {};
