import { CollectionSortProperty } from '../../Runtime/Collection/CollectionSort';
import { EditorIdentifier, FeatureIdentifier } from '@standardnotes/features';
import { SystemViewId } from '../SmartView';
import { TagPreferences } from '../Tag';
export declare enum PrefKey {
    TagsPanelWidth = "tagsPanelWidth",
    NotesPanelWidth = "notesPanelWidth",
    EditorWidth = "editorWidth",
    EditorLeft = "editorLeft",
    EditorMonospaceEnabled = "monospaceFont",
    EditorSpellcheck = "spellcheck",
    EditorResizersEnabled = "marginResizersEnabled",
    EditorLineHeight = "editorLineHeight",
    EditorLineWidth = "editorLineWidth",
    EditorFontSize = "editorFontSize",
    SortNotesBy = "sortBy",
    SortNotesReverse = "sortReverse",
    NotesShowArchived = "showArchived",
    NotesShowTrashed = "showTrashed",
    NotesHideProtected = "hideProtected",
    NotesHidePinned = "hidePinned",
    NotesHideNotePreview = "hideNotePreview",
    NotesHideDate = "hideDate",
    NotesHideTags = "hideTags",
    NotesHideEditorIcon = "hideEditorIcon",
    UseSystemColorScheme = "useSystemColorScheme",
    AutoLightThemeIdentifier = "autoLightThemeIdentifier",
    AutoDarkThemeIdentifier = "autoDarkThemeIdentifier",
    NoteAddToParentFolders = "noteAddToParentFolders",
    NewNoteTitleFormat = "newNoteTitleFormat",
    CustomNoteTitleFormat = "customNoteTitleFormat",
    UpdateSavingStatusIndicator = "updateSavingStatusIndicator",
    DefaultEditorIdentifier = "defaultEditorIdentifier",
    MomentsDefaultTagUuid = "momentsDefaultTagUuid",
    ClipperDefaultTagUuid = "clipperDefaultTagUuid",
    SystemViewPreferences = "systemViewPreferences",
    SuperNoteExportFormat = "superNoteExportFormat",
    AuthenticatorNames = "authenticatorNames",
    PaneGesturesEnabled = "paneGesturesEnabled"
}
export declare enum NewNoteTitleFormat {
    CurrentDateAndTime = "CurrentDateAndTime",
    CurrentNoteCount = "CurrentNoteCount",
    CustomFormat = "CustomFormat",
    Empty = "Empty"
}
export declare enum EditorLineHeight {
    None = "None",
    Tight = "Tight",
    Snug = "Snug",
    Normal = "Normal",
    Relaxed = "Relaxed",
    Loose = "Loose"
}
export declare enum EditorLineWidth {
    Narrow = "Narrow",
    Wide = "Wide",
    Dynamic = "Dynamic",
    FullWidth = "FullWidth"
}
export declare enum EditorFontSize {
    ExtraSmall = "ExtraSmall",
    Small = "Small",
    Normal = "Normal",
    Medium = "Medium",
    Large = "Large"
}
export type PrefValue = {
    [PrefKey.TagsPanelWidth]: number;
    [PrefKey.NotesPanelWidth]: number;
    [PrefKey.EditorWidth]: number | null;
    [PrefKey.EditorLeft]: number | null;
    [PrefKey.EditorMonospaceEnabled]: boolean;
    [PrefKey.EditorSpellcheck]: boolean;
    [PrefKey.EditorResizersEnabled]: boolean;
    [PrefKey.SortNotesBy]: CollectionSortProperty;
    [PrefKey.SortNotesReverse]: boolean;
    [PrefKey.NotesShowArchived]: boolean;
    [PrefKey.NotesShowTrashed]: boolean;
    [PrefKey.NotesHidePinned]: boolean;
    [PrefKey.NotesHideProtected]: boolean;
    [PrefKey.NotesHideNotePreview]: boolean;
    [PrefKey.NotesHideDate]: boolean;
    [PrefKey.NotesHideTags]: boolean;
    [PrefKey.NotesHideEditorIcon]: boolean;
    [PrefKey.UseSystemColorScheme]: boolean;
    [PrefKey.AutoLightThemeIdentifier]: FeatureIdentifier | 'Default' | 'Dark';
    [PrefKey.AutoDarkThemeIdentifier]: FeatureIdentifier | 'Default' | 'Dark';
    [PrefKey.NoteAddToParentFolders]: boolean;
    [PrefKey.NewNoteTitleFormat]: NewNoteTitleFormat;
    [PrefKey.CustomNoteTitleFormat]: string;
    [PrefKey.EditorLineHeight]: EditorLineHeight;
    [PrefKey.EditorLineWidth]: EditorLineWidth;
    [PrefKey.EditorFontSize]: EditorFontSize;
    [PrefKey.UpdateSavingStatusIndicator]: boolean;
    [PrefKey.DefaultEditorIdentifier]: EditorIdentifier;
    [PrefKey.MomentsDefaultTagUuid]: string | undefined;
    [PrefKey.ClipperDefaultTagUuid]: string | undefined;
    [PrefKey.SystemViewPreferences]: Partial<Record<SystemViewId, TagPreferences>>;
    [PrefKey.SuperNoteExportFormat]: 'json' | 'md' | 'html';
    [PrefKey.AuthenticatorNames]: string;
    [PrefKey.PaneGesturesEnabled]: boolean;
};
