import { AnyRecord } from '@standardnotes/common';
export declare function getGlobalScope(): Window | unknown | null;
export declare function dictToArray<T>(dict: Record<any, T>): T[];
/**
 * Whether we are in a web browser
 */
export declare function isWebEnvironment(): boolean;
/**
 * @returns true if WebCrypto is available
 */
export declare function isWebCryptoAvailable(): boolean;
/**
 * Whether we are in React Native app
 */
export declare function isReactNativeEnvironment(): boolean;
/**
 * Searches array of objects for first object where object[key] === value
 * @returns Matching object or null if not found
 */
export declare function findInArray<T, K extends keyof T>(array: T[], key: K, value: T[K]): T | undefined;
/**
 * Searches array of objects for first object where object[key] === value
 * @returns Matching object or null if not found
 */
export declare function searchArray<T>(array: T[], predicate: Partial<T>): T | undefined;
export declare function sureSearchArray<T>(array: T[], predicate: Partial<T>): T;
/**
 * @returns Whether the value is a function or object
 */
export declare function isObject(value: unknown): value is object;
/**
 * @returns Whether the value is a function
 */
export declare function isFunction(value: unknown): boolean;
/**
 * @returns True if the object is null or undefined, otherwise false
 */
export declare function isNullOrUndefined(value: unknown): value is null | undefined;
export declare function isNotUndefined<T>(val: T | undefined | null): val is T;
/**
 * @returns True if the string is empty or undefined
 */
export declare function isEmpty(string: string): boolean;
/**
 * @returns Whether the value is a string
 */
export declare function isString(value: unknown): value is string;
/**
 * @returns The greater of the two dates
 */
export declare function greaterOfTwoDates(dateA: Date, dateB: Date): Date;
/**
 * Returns a new array containing only unique values by combining the two input arrays.
 * Elements are unique based on the values of `equalityKeys`.
 * @param equalityKeys - Keys to determine element equality
 * @returns Array containing unique values
 */
export declare function uniqCombineObjArrays<T>(arrayA: T[], arrayB: T[], equalityKeys: (keyof T)[]): T[];
/**
 * Returns a new array containing only unique values
 * @returns Array containing unique values
 */
export declare function uniqueArray<T>(array: T[]): T[];
/**
 * Returns a new array containing only unique values
 * @returns Array containing unique values
 */
export declare function uniqueArrayByKey<T>(array: T[], key: keyof T): T[];
/**
 * Returns the last element in the array.
 * @returns The last element in the array
 */
export declare function lastElement<T>(array: T[]): T | undefined;
/**
 * Adds all items from otherArray into inArray, in-place.
 * Does not return a value.
 */
export declare function extendArray<T, K extends T>(inArray: T[], otherArray: K[]): void;
/**
 * Removes all items appearing in toSubtract from inArray, in-place
 * @param toSubtract - The list of items to remove from inArray
 */
export declare function subtractFromArray<T>(inArray: T[], toSubtract: T[]): void;
/**
 * Removes the first matching element of an array by strict equality.
 * If no matchin element is found, the array is left unchanged.
 */
export declare function removeFromArray<T>(array: T[], value: T): void;
/**
 * Adds the element to the array if the array does not already include the value.
 * The array is searched via array.indexOf
 * @returns true if value was added
 */
export declare function addIfUnique<T>(array: T[], value: T): boolean;
/**
 * Removes an object from the array in-place by searching for an object where all the
 * key/values in predicate match with the candidate element.
 */
export declare function filterFromArray<T>(array: T[], predicate: Partial<Record<keyof T, any>> | ((object: T) => boolean)): void;
/**
 * Returns a new array by removing all elements in subtract from array
 */
export declare function arrayByDifference<T>(array: T[], subtract: T[]): T[];
export declare function compareValues<T>(left: T, right: T): boolean;
/**
 * Removes the value from the array at the given index, in-place.
 */
export declare function removeFromIndex(array: any[], index: number): void;
/**
 * Adds the value from the array at the given index, in-place.
 */
export declare function addAtIndex<T>(array: T[], element: T, index: number): void;
/**
 * Returns a new array by removeing the value from the array at the given index
 */
export declare function arrayByRemovingFromIndex<T>(array: T[], index: number): T[];
/**
 * Returns an array where each element is the value of a top-level
 * object key.
 * Example: objectToValueArray({a: 1, b: 2}) returns [1, 2]
 */
export declare function objectToValueArray(object: AnyRecord): any[];
/**
 * Returns a key-sorted copy of the object.
 * For example, sortedCopy({b: '1', a: '2'}) returns {a: '2', b: '1'}
 */
export declare function sortedCopy(object: any): any;
export declare const sortByKey: <T>(input: T[], key: keyof T) => T[];
/** Returns a new object by omitting any keys which have an undefined or null value  */
export declare function omitUndefinedCopy(object: any): any;
/**
 * Returns a new array by sorting an array of elements based on a date property,
 * as indicated by the input key value.
 */
export declare function dateSorted<T>(elements: T[], key: keyof T, ascending?: boolean): T[];
/** Compares for equality by comparing top-level keys value equality (===) */
export declare function topLevelCompare<T>(left: T, right: T): boolean;
/**
 * Returns a new object by attempting to JSON.parse any top-level object keys.
 */
export declare function jsonParseEmbeddedKeys(object: AnyRecord): Partial<Record<string, any>>;
export declare const withoutLastElement: <T>(array: T[]) => T[];
/**
 * Deletes keys of the input object.
 */
export declare function omitInPlace<T>(object: T, keys: Array<keyof T>): void;
/**
 * Creates a new object by omitting `keys` from `object`
 */
export declare function omitByCopy<T>(object: T, keys: Array<keyof T>): ({} & T) | undefined;
/**
 * Similiar to Node's path.join, this function combines an array of paths into
 * one resolved path.
 */
export declare function joinPaths(...args: string[]): string;
/**
 * Creates a copy of the input object by JSON stringifying the object then JSON parsing
 * the string (if the input is an object). If input is date, a Date copy will be created,
 * and if input is a primitive value, it will be returned as-is.
 */
export declare function Copy(object: any): any;
/**
 * Merges the second object parameter into the first object, in-place.
 * @returns The now modified first object parameter passed into the function.
 */
export declare function deepMerge(a: AnyRecord, b: AnyRecord): Partial<Record<string, any>>;
/**
 * Returns a new object by selecting certain keys from input object.
 */
export declare function pickByCopy<T>(object: T, keys: Array<keyof T>): any;
/**
 * Recursively makes an object immutable via Object.freeze
 */
export declare function deepFreeze(object: any): any;
export declare function isValidUrl(url: string): boolean;
/**
 * Determines if an object has a getter defined for a given property
 */
export declare function hasGetter(object: any, property: string): boolean | undefined;
/**
 * Truncates a hex string into a desired number of bits
 * @returns A hexadecimal string truncated to the number of desired bits
 */
export declare function truncateHexString(string: string, desiredBits: number): string;
/**
 * When awaited, this function allows code execution to pause for a set time.
 * Should be used primarily for testing.
 */
export declare function sleep(milliseconds: number, warn?: boolean, desc?: string): Promise<void>;
export declare function assertUnreachable(uncheckedCase: never): never;
/**
 * Returns a boolean representing whether two dates are on the same day
 */
export declare function isSameDay(dateA: Date, dateB: Date): boolean;
/**
 * Sorts an array of objects in natural order
 * @param items - The array of objects to sort
 * @param property - The objects' property to sort by
 * @param direction - The sorting direction, either ascending (default) or descending
 * @returns Array of objects sorted in natural order
 */
export declare function naturalSort<T extends AnyRecord>(items: T[], property: keyof T, direction?: 'asc' | 'desc'): T[];
export declare function arraysEqual<T>(left: T[], right: T[]): boolean;
export declare function convertTimestampToMilliseconds(timestamp: number): number;
export declare function sanitizeHtmlString(html: string): string;
export declare function dateToLocalizedString(date: Date): string;
export declare function nonSecureRandomIdentifier(): string;
export declare function splitString(string: string, parts: number): string[];
export declare function firstHalfOfString(string: string): string;
export declare function secondHalfOfString(string: string): string;
export declare function log(namespace: string, ...args: any[]): void;
export declare function logWithColor(namespace: string, namespaceColor: string, ...args: any[]): void;
export declare function assert(value: unknown): asserts value;
export declare function useBoolean(value: boolean | undefined, defaultValue: boolean): boolean;
export declare function spaceSeparatedStrings(...strings: string[]): string;
export declare function pluralize(count: number, singular: string, plural: string): string;
